/*!
 * objectiveVR - html and javascript based, plugin-free object vr viewer for your browser
 * http://9tm.de/code/ObjectiveVR/
 *
 * Copyright 2010-2013 MikeMyDay (Michael Hermes)
 * http://www.mikemyday.de/
 *
 * You can use, modify and distribute objectiveVR, but do so in the spirit of Open Source.
 * You need to leave the copyright notices intact.
 * You need to be able to share any modifications you make to the objectiveVR source
 * (not the software objectiveVR is integrated to).
 * Give credit where credit is due, spread the word, link to me if you can.
 * 
 * This file is part of objectiveVR.
 * 
 * objectiveVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * objectiveVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Requires jQuery JavaScript Library v1.4.2
 * http://jquery.com/
 * Copyright 2010, John Resig
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 * Or link the google-hosted version:
 * <script type="text/javascript" src="http://ajax.googleapis.com/ajax/libs/jquery/1.4.2/jquery.min.js"></script>
 * Is compatible with the jQuery.noConflict() Option.
 */

function ObjectiveVR(settings){
	//init:
	var self=this;
		
	this.initializeWithSettings = function (settings){
		//stopping redraw
		window.clearInterval(self.autoAnimationInterval);
		window.clearInterval(self.manualAnimationInterval);
		window.clearInterval(self.scrubbingInterval);
		
		//stopping animation
		self.stopAutoAnimation();
		self.stopManualAnimation();
		
		//initializing variable defaults
		self.imagesDomElements=new Array();
		self.scrubbingStartingPosition=new Object();
		self.motionBlurImagesOrderArray=new Array();
		self.viewportObject=false;
		self.manualAnimationInterval=false;
		self.viewportWidth=false;
		self.viewportHeight=false;
		self.controlsObject=false;
		self.scrubbingControlObject=false;
		self.scrubbingInterval=false;
		self.scrubbingActive=false;
		self.autoAnimationInterval=false;
		self.matchingImageNumberX=0;
		self.matchingImageNumberY=0;
		self.scrubbingDeltaX=0;
		self.motionBlurFadeOutTime=0;
		self.framesPerSecond=30;
		self.autoAnimationDegreesPerSecond=30;
		self.manualAnimationDegreesPerSecond=90;
		self.autoAnimateStep=this.autoAnimationDegreesPerSecond/this.framesPerSecond;
		self.manualAnimationStep=this.manualAnimationDegreesPerSecond/this.framesPerSecond;
	
		self.settings=settings;
	
		self.currentImageCounterX=-1;
		self.currentImageCounterY=0;
		
		if (self.settings.viewport!="unknown"){
			jQuery(document).ready(function(){
				if ('object'==typeof(jQuery('#'+self.settings.viewport))){
					self.viewportObject=jQuery('#'+self.settings.viewport);
					self.initialize();	
				}
			});	
		}
	}
	
	this.initialize = function (){		
		self.viewportObject.empty();
		self.viewportObject.addClass('objectiveVrViewport');
		self.viewportObject.css('position','relative');
		self.viewportObject.prepend('<div class="progressBarContainer"><div class="progress"></div></div><div class="imagesPool"></div>');
		self.imagesPoolElement=self.viewportObject.find('div.imagesPool');
		self.viewportObject.append('<span class="navigator3d"></span>');
		self.viewportObject.find('div.progressBarContainer').fadeIn(100);
		
		self.tryToLoadNextImage();
	}

	//load images
	this.tryToLoadNextImage = function (){	
		if (typeof(self.settings.imagesCountX)!='undefined' && typeof(self.settings.imagesCountY)!='undefined'){
			if(0==self.settings.imagesCountX)self.settings.imagesCountX=1;
			if(0==self.settings.imagesCountY)self.settings.imagesCountY=1;
			var imagesToBeLoaded=self.settings.imagesCountY*self.settings.imagesCountX;
			var imagesLoaded=self.currentImageCounterX+(self.currentImageCounterY*self.settings.imagesCountX);
			var percentComplete=imagesLoaded/imagesToBeLoaded*100;
			self.viewportObject.find('.progressBarContainer .progress').css('width',percentComplete+'%'); //no longer displaying the abs number of images: .html(imagesLoaded+'/'+imagesToBeLoaded);
		}
		
		if (typeof(self.settings.imagesCountX)!='undefined' && self.currentImageCounterX>=self.settings.imagesCountX-1){
			self.currentImageCounterX=-1;
			self.currentImageCounterY++;
		}
		if (typeof(self.settings.imagesCountX)=='undefined' || self.currentImageCounterX<self.settings.imagesCountX-1){
			//move ahead on the x axis
			self.currentImageCounterX++;
		}else if (typeof(self.settings.imagesCountY)=='undefined' || self.currentImageCounterY<self.settings.imagesCountY-1){
			//reset x to the first image and move up onto the next y layer
			self.currentImageCounterY++;
		}
		
		var currentImageFilename=self.settings.filenamePattern.replace("%COUNTX%",self.currentImageCounterX).replace("%COUNTY%",self.currentImageCounterY);
		self.currentImage=new Image();
		self.currentImage.src=currentImageFilename;
		
		if (self.currentImage.complete){
			if(self.currentImage.height>0){
				self.appendImageToQueue();
			}else{
				self.imageNotFound();
			}
		}else{
			jQuery(self.currentImage).bind('abort',self.imageLoadingAborted);	
			jQuery(self.currentImage).bind('error',self.imageNotFound);	
			jQuery(self.currentImage).bind('load', self.appendImageToQueue);
		}
	}
	
	this.appendImageToQueue = function (){
		if (typeof(self.imagesDomElements[self.currentImageCounterX])!='object')self.imagesDomElements[self.currentImageCounterX]=new Array();
		self.imagesDomElements[self.currentImageCounterX][self.currentImageCounterY]=(jQuery('<img src="'+self.settings.filenamePattern.replace("%COUNTX%",self.currentImageCounterX).replace("%COUNTY%",self.currentImageCounterY)+'" />').css('display','hide').appendTo(self.imagesPoolElement));
		if (typeof(self.settings.imagesCountX)!='undefined' && self.currentImageCounterX>=self.settings.imagesCountX-1 && typeof(self.settings.imagesCountY)!='undefined' && self.currentImageCounterY>=self.settings.imagesCountY-1){
			//we reached the highest x and y position. let's init the display
			self.initDisplay();
		}else{
			self.tryToLoadNextImage();
		}
	}

	this.imageNotFound = function (){
		if (0==self.currentImageCounterX && 0==self.currentImageCounterY){
			//the root image could not be found. bad times, maybe there's an error in the settings.
			self.viewportObject.empty().append('<div class="errorMessage">Could not even load the first image (x=0 + y=0).<br />Please check path and filename in your settings.</div>');
		}else{
			if (self.currentImageCounterX!=0){
				//we found the highest avilable image number on the x axis
				self.settings.imagesCountX=self.currentImageCounterX;
				//there may be another y axis, though. so let's try to load another image
				self.tryToLoadNextImage();
			}else{
				//we found the highest avilable image number on the y axis
				self.settings.imagesCountY=self.currentImageCounterY;
				//since we're checking every x on every y axis this means we've found all images
				//let's init the display
				self.initDisplay();
			}
		}
	}
	
	this.imageLoadingAborted = function (){
		window.status('image loading aborted ...');
		self.imageNotFound();
	}

	//display the images
	this.initDisplay = function (){
		self.viewportObject.find('.progressBarContainer .progress').css('width','100%').parent().fadeOut(250);
		//setting default maxAngleY value, guessing on number of viewport angles found for the y axis
		if (typeof(self.settings.maxAngleY)=='undefined')Math.floor(self.settings.maxAngleY=(self.settings.imagesCountY-1)*22.5);
		if(self.settings.maxAngleY>90)self.settings.maxAngleY=90;
		if(self.settings.maxAngleY<1)self.settings.maxAngleY=1;
		
		self.degreesPerFrameX=360/(self.settings.imagesCountX);
		self.degreesPerFrameY=self.settings.maxAngleY/(self.settings.imagesCountY);
		//making sure these will not be 0 or else we'll get a division by zero later on...
		if (0>=self.degreesPerFrameX)self.degreesPerFrameX=.01;
		if (0>=self.degreesPerFrameY)self.degreesPerFrameY=.01;
		
		self.currentAngleX=0;
		self.currentAngleY=Math.floor(self.settings.maxAngleY/2);
		self.updateDisplay();
		
		self.viewportWidth=self.viewportObject.width();
		self.viewportHeight=self.viewportObject.height();
		
		//InitMotionBlur
		if (typeof(self.settings.motionBlur)=='undefined')self.settings.motionBlur=0;
		if (self.settings.motionBlur>20)self.settings.motionBlur=20;
		if (self.settings.motionBlur<0)self.settings.motionBlur=0;
		self.motionBlurFadeOutTime=self.settings.motionBlur*(1000/30);
		
		if (self.settings.autoAnimate==true){
			self.startAutoAnimation();
		}
		self.initControls();
	}

	this.initControls = function(){
		//Mouse/Touch Controls:
		self.viewportObject.prepend('<div class="overlay"></div>');
		this.scrubbingControlObject=self.viewportObject.find('.overlay');
		this.scrubbingControlObject.bind('mousedown touchstart',function(e){
			e.preventDefault();
			self.stopAutoAnimation();
			self.startScrubbing(e);
			return(false);
		});
		self.viewportObject.find('.overlay').bind('mouseup touchend mouseout',function(e){
			e.preventDefault();
			self.stopScrubbing(e);
			return(false);
		});
		
		if(self.settings.imagesCountY>1){
			var buttonsY='<a class="moveNorthButton"></a><a class="moveSouthButton"></a>';
		}else{
			var buttonsY='';
		}
		//Button Controls:
		self.controlsObject=self.viewportObject.append('<div class="controls"><a class="playPauseButton"></a><a class="moveEastButton"></a><a class="moveWestButton"></a>'+buttonsY+'</div>');
		self.controlsObject.find('.playPauseButton').bind('click',function(e){
			e.preventDefault();
			self.stopManualAnimation();
			if(self.autoAnimationInterval==false){
				self.settings.autoAnimate=true;
				self.startAutoAnimation();
			}else{
				self.settings.autoAnimate=false;
				self.stopAutoAnimation();
			}
			return(false);
		});
		
		self.controlsObject.find('.moveWestButton').bind('mousedown touchstart',function(e){
			e.preventDefault();
			self.stopAutoAnimation();
			self.startManualAnimation(0-self.manualAnimationDegreesPerSecond,'xAxis');
			return(false);
		}).bind('mouseup mouseout touchend',function(e){
			e.preventDefault();
			self.stopManualAnimation();
			return(false);
		});
		self.controlsObject.find('.moveEastButton').bind('mousedown touchstart',function(e){
			self.stopAutoAnimation();
			self.startManualAnimation(0+self.manualAnimationDegreesPerSecond,'xAxis');
			e.preventDefault();
			return(false);
		}).bind('mouseup mouseout touchend',function(e){
			e.preventDefault();
			self.stopManualAnimation();
			return(false);
		});
		self.controlsObject.find('.moveNorthButton').bind('mousedown touchstart',function(e){
			e.preventDefault();
			self.stopAutoAnimation();
			self.startManualAnimation(0-self.manualAnimationDegreesPerSecond,'yAxis');
			return(false);
		}).bind('mouseup mouseout touchend',function(e){
			e.preventDefault();
			self.stopManualAnimation();
			return(false);
		});
		self.controlsObject.find('.moveSouthButton').bind('mousedown touchstart',function(e){
			e.preventDefault();
			self.stopAutoAnimation();
			self.startManualAnimation(0+self.manualAnimationDegreesPerSecond,'yAxis');
			return(false);
		}).bind('mouseup mouseout touchend',function(e){
			e.preventDefault();
			self.stopManualAnimation();
			return(false);
		});	
	}
	
	//main display-update function
	this.updateDisplay = function (){
		var imgBufferX=self.matchingImageNumberX;
		var imgBufferY=self.matchingImageNumberY;
		if (self.currentAngleX>=360)self.currentAngleX-=360;
		if (self.currentAngleX<0)self.currentAngleX+=360;
		if (self.currentAngleY>self.settings.maxAngleY-.1)self.currentAngleY=self.settings.maxAngleY-.1;
		if (self.currentAngleY<0)self.currentAngleY=0;
		self.matchingImageNumberX=Math.floor(self.currentAngleX/self.degreesPerFrameX);		
		self.matchingImageNumberY=Math.floor(self.currentAngleY/self.degreesPerFrameY);
		//current image changed. updateing display
		if (imgBufferX!=self.matchingImageNumberX || imgBufferY!=self.matchingImageNumberY){
			
//			window.status='X: '+self.matchingImageNumberX+'/'+self.imagesDomElements.length+'-Y: '+self.matchingImageNumberY+'/'+self.imagesDomElements[self.matchingImageNumberX].length;
			
			self.imagesDomElements[self.matchingImageNumberX][self.matchingImageNumberY].stop(true,true).show();
			if (self.settings.motionBlur>0){
				self.motionBlurImagesOrderArray.unshift(self.imagesDomElements[self.matchingImageNumberX][self.matchingImageNumberY]);
				if (self.motionBlurImagesOrderArray.length>self.settings.motionBlur)self.motionBlurImagesOrderArray.pop();
				for (var i=self.motionBlurImagesOrderArray.length-1;i>=0;i--){
					self.motionBlurImagesOrderArray[i].css('z-index',i);
				}
				self.imagesDomElements[imgBufferX][imgBufferY].stop(true,true).fadeOut(self.motionBlurFadeOutTime);
			}else{
				self.imagesDomElements[imgBufferX][imgBufferY].hide();
			}
		}
		var compassStep=Math.floor(self.currentAngleX/10);
		self.viewportObject.find('span.navigator3d').css('background-position','-'+(compassStep*50)+'px');
	}
	
	//Automatic Playback
	this.startAutoAnimation = function(){
		self.stopAutoAnimation();
		self.autoAnimationInterval=window.setInterval(function(){
			self.doAutoAnimation();
		},1000/self.framesPerSecond);	
	}
	this.stopAutoAnimation = function(){
		window.clearInterval(self.autoAnimationInterval);
		self.autoAnimationInterval=false;
	}
	this.doAutoAnimation = function(){
		self.currentAngleX+=self.autoAnimateStep;
		self.updateDisplay();
	}
	
	//Manual Playback
	this.startManualAnimation = function (degreesPerSecond,axis){
		if (degreesPerSecond!=0){
			self.manualAnimateStep=degreesPerSecond/self.framesPerSecond;
		}
		self.stopManualAnimation();
		//immediately move to the next viewpoint on that axis, to give a better response to the user
		if (axis=='xAxis'){
			if (degreesPerSecond>0){
				self.currentAngleX=(self.matchingImageNumberX*self.degreesPerFrameX);
			}else{
				self.currentAngleX=(self.matchingImageNumberX*self.degreesPerFrameX)+self.degreesPerFrameX;
			}
		}else{
			if (degreesPerSecond>0){
				self.currentAngleY=(self.matchingImageNumberY*self.degreesPerFrameY);
			}else{
				self.currentAngleY=(self.matchingImageNumberY*self.degreesPerFrameY)+self.degreesPerFrameY;
			}
		}
		//now start the loop for continous movement in that direction
		self.manualAnimationInterval=window.setInterval(function(){
			self.doManualAnimation(axis);
		},1000/this.framesPerSecond);
	}	
	this.stopManualAnimation = function (){
		window.clearInterval(self.manualAnimationInterval);
		manualAnimationInterval=false;
	}
	this.doManualAnimation = function(axis){
		if (axis=='xAxis'){
			self.currentAngleX-=self.manualAnimateStep;
		}else{
			self.currentAngleY-=self.manualAnimateStep/(360/this.settings.maxAngleY);
		}
		self.updateDisplay();
	}
	
	//Mouse Scrubbing
	this.startScrubbing = function(e){
		var oe=e.originalEvent;
		self.scrubbingDeltaX=0;
		self.scrubbingDeltaY=0;
		self.scrubbingActive=true;
		var offset=self.scrubbingControlObject.offset();
		if(typeof(oe.touches)=='undefined'){
			self.scrubbingStartingPosition.x=(e.pageX-offset.left)/self.viewportWidth*360;
			self.scrubbingStartingPosition.y=(e.pageY-offset.top)/self.viewportHeight*360;
			self.viewportObject.bind('mousemove',self.captureMouse);
		}else{
			self.scrubbingStartingPosition.x=(oe.touches[0].pageX-offset.left)/self.viewportWidth*360;
			self.scrubbingStartingPosition.y=(oe.touches[0].pageY-offset.top)/self.viewportHeight*360;
			self.viewportObject.bind('touchmove',self.captureTouch);
		}
		window.clearInterval(self.scrubbingInterval);
		self.scrubbingInterval=window.setInterval(function(){
			self.doScrubbing();
		},1000/self.framesPerSecond);	
	}
	this.stopScrubbing = function (e){
		var oe=e.originalEvent;
		self.scrubbingActive=false;
		if(typeof(oe.touches)=='undefined'){
			self.viewportObject.unbind('mousemove',self.captureMouse);
		}else{
			self.viewportObject.unbind('touchmove',function(e){self.captureTouch(e)});
		}
	}
	//handle pointer input devices:
	this.captureMouse = function (e){
		var offset=self.scrubbingControlObject.offset();
		self.scrubbingDeltaX=-1*(self.scrubbingStartingPosition.x-((e.pageX-offset.left)/self.viewportWidth*360));
		self.scrubbingDeltaY=-1*(self.scrubbingStartingPosition.y-((e.pageY-offset.top)/self.viewportHeight*360));
		return(false);
	}
	//handle iOS touch input devices
	this.captureTouch = function(e){
		e=e.originalEvent;
		var offset=self.scrubbingControlObject.offset();
		if(e.touches.length == 1){
			if (jQuery(e.touches[0].target).attr('class')=='overlay'){
				self.scrubbingDeltaX=-1.6*(self.scrubbingStartingPosition.x-((e.touches[0].pageX-offset.left)/self.viewportWidth*360));
				self.scrubbingDeltaY=-1.6*(self.scrubbingStartingPosition.y-((e.touches[0].pageY-offset.top)/self.viewportHeight*360));
			}
			e.preventDefault();
			return(false);
		}
	}

	this.doScrubbing = function (){
		//limiting movement. mouse delta must be alt least 32px for any axis before the viewpoint will be moved accordingly
		if (self.scrubbingDeltaX>=-10 && self.scrubbingDeltaX<=10)self.scrubbingDeltaX=0;
		if (self.scrubbingDeltaY>=-10 && self.scrubbingDeltaY<=10)self.scrubbingDeltaY=0;
		if (!self.scrubbingActive){
			self.scrubbingDeltaX=self.scrubbingDeltaX*0.9;
			self.scrubbingDeltaY=self.scrubbingDeltaY*0.9;
			
			if (self.scrubbingDeltaX==0 && self.scrubbingDeltaY==0){
				window.clearInterval(self.scrubbingInterval);
				self.scrubbingInterval=false;
			}
		}
		self.currentAngleX+=(self.scrubbingDeltaX/25);
		//movement on the y axis should be much slower than on the x axis, because we're only using 0 to 90° instead of 0 to 360° on the x axis
		self.currentAngleY+=(self.scrubbingDeltaY/(25*(360/this.settings.maxAngleY)));
		self.updateDisplay();
	}

	//initialize and start up
	this.initializeWithSettings(settings);
}